unit tileunit;

interface

uses SysUtils, Classes, Graphics, util;

type
  // Ϥݩ: iHqL, ϤiQ}, luiHqL, ϤOxX
  TTileAttr = set of (taCanPass, taCanBreak, taBulletCanPass, taFlag);
  
  TTile = class // @Ϥ
  private
    FBits: TBitmap; // sϤ bitmap
    FAttr: TTileAttr; // ݩ
    FDisposed: Boolean; // O_wm
    
    FFirstTile: Boolean; // OOϤsո̪Ĥ@iϤ
    FXNum, FYNum: Byte; // ϤsժVaVϤƥ
  protected
  public
    constructor Create;
    destructor Destroy; override;
    
    // JxsϤ
    procedure LoadFromStream(Stream: TStream);
    procedure SaveToStream(Stream: TStream);
    
    // ѵ~ɦsݩ
    property Bitmap: TBitmap read FBits;
    property Attr: TTileAttr read FAttr write FAttr;
    property Disposed: Boolean read FDisposed write FDisposed;
    
    property FirstTile: Boolean read FFirstTile write FFirstTile;
    property xNum: Byte read FXNum write FXNum;
    property yNum: Byte read FYNum write FYNum;
  end;
  
  TTiles = class // Ϯw
  private
    FTiles: array of TTile; // Ҿ֦Ϥ}C
    
    function GetTileNum: Integer; // Ϥƥ
    
    function GetTile(No: Integer): TTile; // QίިoϤ
  protected
  public
    constructor Create;
    destructor Destroy; override;
    
    function AddTile(NewTile: TTile): Integer; // [JsϤ
    procedure FreeTiles; // ҦϤ
    
    // JxsϮw
    procedure LoadFromFile(Filename: string);
    procedure SaveToFile(Filename: string);
    
    property TileNum: Integer read GetTileNum;
    property Tile[No: Integer]: TTile read GetTile; default; // w]}Cݩ
  end;
  
var
  Tiles: TTiles; // @ӹϮwNF
  
implementation

constructor TTile.Create;
begin
  inherited Create;
  
  // xϤ bitmap, jp TILE_WIDTH * TILE_HEIGHT
  FBits := TBitmap.Create;
  FBits.Width := TILE_WIDTH;
  FBits.Height := tile_height;
  FBits.Transparent := True;
  FBits.TransparentColor := TRANSPARENT_COLOR;
  
  // w]iϤΦ@Ӹs
  FFirstTile := True;
  FXNum := 1;
  FYNum := 1;
end;

destructor TTile.Destroy;
begin
  FBits.Free;
  
  inherited Destroy;
end;

procedure TTile.LoadFromStream(Stream: TStream);
begin
  with Stream do
  begin
    read (FDisposed, sizeof(Boolean));
    // O_wm ?? OܴNAŪ䥦ݩ
    if FDisposed then Exit;

    FBits.LoadFromStream(Stream); // ϧ
    read (FAttr, sizeof(TTileAttr)); // ݩ
    
    read (FFirstTile, sizeof(Boolean));
    
    if FFirstTile then // YOsYY, hŪsժe
    begin
      read (FXNum, sizeof(Byte));
      read (FYNum, sizeof(Byte));
    end;
  end;
end;

procedure TTile.SaveToStream(Stream: TStream);
begin
  with Stream do
  begin
    write (FDisposed, sizeof(Boolean));
    // O_wm ?? OܴNAgJ䥦ݩ
    if FDisposed then Exit;
    
    FBits.SaveToStream(Stream); // ϧ
    write (FAttr, sizeof(TTileAttr)); // ݩ
    
    write (FFirstTile, sizeof(Boolean));
    
    if FFirstTile then // YOsYY, hgJsժe
    begin
      write (FXNum, sizeof(Byte));
      write (FYNum, sizeof(Byte));
    end;
  end;
end;

{ TTiles class }

constructor TTiles.Create;
begin
  inherited Create;
  
  SetLength(FTiles, 0);
end;

destructor TTiles.Destroy;
begin
  FreeTiles;
  
  inherited Destroy;
end;

// [JsϤ, b}C̫
function TTiles.AddTile(NewTile: TTile): Integer;
begin
  SetLength(FTiles, Length(FTiles) + 1);
  FTiles[Length(FTiles) - 1] := NewTile;
  Result := Length(FTiles) - 1;
end;

// ҦϤ
procedure TTiles.FreeTiles;
var
  I: Integer;
begin
  for I := 0 to Length(FTiles) - 1 do
    FTiles[I].Free;
  SetLength(FTiles, 0);
end;

// qɮŪϮw
procedure TTiles.LoadFromFile(Filename: string);
var
  fs    : TFileStream;
  I, Val: Integer;    
begin
  fs := TFileStream.Create(Filename, fmOpenRead);
  try
    CheckSignature(fs, SIG_MYFILE); // ˬdY
    CheckSignature(fs, ClassName); // ˬdY
    
    fs.read (Val, sizeof(Integer)); // ŪJ tiles ƥ
    if Val = 0 then
      raise Exception.Create('No tiles');
    
    FreeTiles;
    SetLength(FTiles, Val);
    
    for I := 0 to Val - 1 do
    begin
      FTiles[I] := TTile.Create;
      FTiles[I].LoadFromStream(fs); // ŪJҦ tiles
    end;
  finally
    fs.Free;
  end;
end;

// gJϮw
procedure TTiles.SaveToFile(Filename: string);
var
  fs: TFileStream;
  S : string;     
  I : Integer;    
begin
  fs := TFileStream.Create(Filename, fmCreate or fmOpenWrite);
  try
    fs.write (SIG_MYFILE, Length(SIG_MYFILE));
    S := ClassName;
    fs.write (S[1], Length(S));
    
    I := TileNum;
    fs.write (I, sizeof(Integer));
    
    for I := 0 to TileNum - 1 do
      FTiles[I].SaveToStream(fs); // gJҦ tiles
  finally
    fs.Free;
  end;
end;

function TTiles.GetTileNum: Integer;
begin
  Result := Length(FTiles);
end;

function TTiles.GetTile(No: Integer): TTile;
begin
  Result := FTiles[No];
end;

end.

